#include "file_writer_filter.hpp"
#include "exception.hpp"
#include "field.hpp"
#include "utils.hpp"
#include "workflow_graph.hpp"

namespace xios
{
  CFileWriterFilter::CFileWriterFilter(CGarbageCollector& gc, CField* field, bool buildWorkflowGraph /* =false */)
    : CInputPin(gc, 1)
    , field(field)
  {
    if (!field)
      ERROR("CFileWriterFilter::CFileWriterFilter(CField* field)",
            "The field cannot be null.");
    if (buildWorkflowGraph)
    {
      if(InvalidableObject::count_ptr == 0) InvalidableObject::count_ptr = new std::map< StdString, int>;
      if(InvalidableObject::count_ptr->find(CWorkflowGraph::my_filename) == InvalidableObject::count_ptr->end())
      {
        (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename] = 0; 
      }
      //filterId = InvalidableObject::count;
      filterId = (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename];
      printf("file_writer_filter ================ filterId = %d\n", (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename]);
      (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename]++;
      //InvalidableObject::count++;
    }
  }

  void CFileWriterFilter::onInputReady(std::vector<CDataPacketPtr> data)
  {
    const bool detectMissingValue = ( !field->default_value.isEmpty() &&
                               ( (!field->detect_missing_value.isEmpty() || field->detect_missing_value == true)
                                 || field->hasGridMask()) );

    CArray<double, 1> dataArray = (detectMissingValue) ? data[0]->data.copy() : data[0]->data;

    if (detectMissingValue)
    {
      const double missingValue = field->default_value;
      const size_t nbData = dataArray.numElements();
      for (size_t idx = 0; idx < nbData; ++idx)
      {
        if (NumTraits<double>::isNan(dataArray(idx)))
          dataArray(idx) = missingValue;
      }
    }

    field->sendUpdateData(dataArray);
  }

  bool CFileWriterFilter::mustAutoTrigger() const
  {
    return true;
  }

  bool CFileWriterFilter::isDataExpected(const CDate& date) const
  {
    return true;
  }

  int CFileWriterFilter::getFilterId(void)
  {
    return filterId;
  }

} // namespace xios
