#include "output_pin.hpp"
#include "exception.hpp"
#include "workflow_graph.hpp"

namespace xios
{
  COutputPin::COutputPin(CGarbageCollector& gc, bool manualTrigger /*= false*/, bool buildWorkflowGraph /* =false */)
    : gc(gc)
    , manualTrigger(manualTrigger)
    , buildWorkflowGraph(buildWorkflowGraph)
  {
    if (buildWorkflowGraph)
    {
      if(InvalidableObject::count_ptr == 0) InvalidableObject::count_ptr = new std::map< StdString, int>;
      if(InvalidableObject::count_ptr->find(CWorkflowGraph::my_filename) == InvalidableObject::count_ptr->end())
      {
        (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename] = 0; 
      }
  
      filterId = (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename];
      printf("output_pin ================ filterId = %d, CWorkflowGraph::my_filename = %s\n", (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename], CWorkflowGraph::my_filename.c_str());
      (*InvalidableObject::count_ptr)[CWorkflowGraph::my_filename]++;

      //filterId = InvalidableObject::count;
      //InvalidableObject::count++;
    }
  }

  StdString COutputPin::GetName(void)
  {
    return StdString("Output pin");
  }

  void COutputPin::connectOutput(std::shared_ptr<CInputPin> inputPin, size_t inputSlot)
  {
    if (!inputPin)
      ERROR("void COutputPin::connectOutput(CInputPin* inputPin, size_t inputSlot)",
            "The input pin cannot be null.");

    outputs.push_back(std::make_pair(inputPin, inputSlot));

    if (canBeTriggered())
      inputPin->setInputTrigger(inputSlot, this);
  }

  void COutputPin::onOutputReady(CDataPacketPtr packet)
  {
    if (!packet)
      ERROR("void COutputPin::onOutputReady(CDataPacketPtr packet)",
            "The packet cannot be null.");

    if (buildWorkflowGraph)
    {
    	if(CWorkflowGraph::mapFilterTimestamps_ptr==0) CWorkflowGraph::mapFilterTimestamps_ptr = new std::unordered_map<StdString, std::unordered_map <int, vector<Time> > >; 
      if(CWorkflowGraph::timestamps_ptr==0) CWorkflowGraph::timestamps_ptr = new std::unordered_map< StdString, set<Time> >;
      (*CWorkflowGraph::mapFilterTimestamps_ptr)[CWorkflowGraph::my_filename][this->getFilterId()].push_back(packet->timestamp);
      (*CWorkflowGraph::timestamps_ptr)[CWorkflowGraph::my_filename].insert(packet->timestamp);
    }

    if (manualTrigger) // Don't use canBeTriggered here, this function is virtual and can be overriden
    {
      outputPackets[packet->timestamp] = packet;
      gc.registerObject(this, packet->timestamp);
    }
    else
      deliverOuput(packet);
  }

  void COutputPin::deliverOuput(CDataPacketPtr packet)
  {
    if (!packet)
      ERROR("void COutputPin::deliverOuput(CDataPacketPtr packet)",
            "The packet cannot be null.");

    std::vector<std::pair<std::shared_ptr<CInputPin>, size_t> >::iterator it, itEnd;
    for (it = outputs.begin(), itEnd = outputs.end(); it != itEnd; ++it)
      it->first->setInput(it->second, packet);
  }

  void COutputPin::trigger(Time timestamp)
  {
    if (manualTrigger) // Don't use canBeTriggered here, this function is virtual and can be overriden
    {
      std::map<Time, CDataPacketPtr>::iterator it = outputPackets.find(timestamp);
      if (it != outputPackets.end())
      {
        gc.unregisterObject(this, timestamp);
        deliverOuput(it->second);
        outputPackets.erase(it);
      }
    }
  }

  bool COutputPin::canBeTriggered() const
  {
    return manualTrigger;
  }

  bool COutputPin::mustAutoTrigger() const
  {
    std::vector<std::pair<std::shared_ptr<CInputPin>, size_t> >::const_iterator it, itEnd;
    for (it = outputs.begin(), itEnd = outputs.end(); it != itEnd; ++it)
    {
      if (it->first->mustAutoTrigger())
        return true;
    }

    return false;
  }

  void COutputPin::setOutputTriggers()
  {
    std::vector<std::pair<std::shared_ptr<CInputPin>, size_t> >::iterator it, itEnd;
    for (it = outputs.begin(), itEnd = outputs.end(); it != itEnd; ++it)
      it->first->setInputTrigger(it->second, this);
  }

  bool COutputPin::isDataExpected(const CDate& date) const
  {
    std::vector<std::pair<std::shared_ptr<CInputPin>, size_t> >::const_iterator it, itEnd;
    for (it = outputs.begin(), itEnd = outputs.end(); it != itEnd; ++it)
    {
      if (it->first->isDataExpected(date))
        return true;
    }

    return false;
  }

  void COutputPin::invalidate(Time timestamp)
  {
    outputPackets.erase(outputPackets.begin(), outputPackets.lower_bound(timestamp));
  }

  int COutputPin::getFilterId(void)
  {
    return filterId;
  }

} // namespace xios
