#ifndef __WORKFLOW_GRAPH_HPP__
#define __WORKFLOW_GRAPH_HPP__

#include "xios_spl.hpp"
#include "field.hpp"
#include "garbage_collector.hpp"

namespace xios
{

  class CWorkflowGraph
  {

    friend class CGraphviz;

    public:

      CWorkflowGraph();

      /*! Map between fields identified by its id and their filters identified by an integer.
       * It is filled up during reconstruction of a workflow (in function CField::buildFilterGraph()).
      */
      static std::unordered_map < StdString, std::unordered_map <StdString, vector <int> >    >  *mapFieldToFilters_ptr;
      #pragma omp threadprivate(mapFieldToFilters_ptr)

      /*! Map between filter ids and filter types.
       * It is filled up during reconstruction of a workflow (in function CField::buildFilterGraph()).
       */
      static std::unordered_map < StdString, std::unordered_map <int, StdString>  >   *mapFilters_ptr;
      #pragma omp threadprivate(mapFilters_ptr)

      /*! Map between filters and timestamps.
       * Timestamps are added into the map at the exit of a filter (in function COutputPin::onOutputReady)
       */
      static std::unordered_map < StdString, std::unordered_map <int, vector<Time> >  > *mapFilterTimestamps_ptr;
      #pragma omp threadprivate(mapFilterTimestamps_ptr)

      /*! List of timestamps.
       * Timestamps are added at the exit of a filter (in function COutputPin::onOutputReady)
       */
      static std::unordered_map< StdString, set<Time> > *timestamps_ptr;
      #pragma omp threadprivate(timestamps_ptr)

      static StdString my_filename;
      #pragma omp threadprivate(my_filename)

    private:

      //! List of fields marked for visualizing of their workflow
      static vector <StdString> fields;

      //! List of connected filters for fields for visualizing of their workflow
      static vector <StdString> filters;

      //! Fields to filters connectivity
      static vector <pair<int, int> > fieldsToFilters;

      static void buildStaticWorkflow();
      static void buildStaticWorkflow_forfile(StdString filename);
      static void showStaticWorkflow();


  };
}

#endif

